/*******************************************************************************
 *	BdsCreateMetaData.cc	Creates BDS test Meta Data
 *				T.Barnaby,	BEAM Ltd,	2009-10-12
 *******************************************************************************
 *
 * This program creates the test MetaData in the BDS system.
 */
#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <errno.h>
#include <BdsD.h>
#include <BdsC.h>
#include <BFile.h>
#include <BdsLib.h>
#include <sys/stat.h>

using namespace std;
using namespace Bds;

class Config {
public:
				Config();
	int			verbose;
};

Config::Config(){
	verbose = 0;
}

BError testDeleteMetaData(AdminAccess& bds, Config& config){
	BError				err;
	Selection			sel;
	BList<Network>			networks;
	BList<Station>			stations;
	BList<Location>			locations;
	BList<Channel>			channels;
	BList<ChannelInstrument>	channelInstruments;
	BList<Response>			responses;
	BList<Calibration>		calibrations;
	BIter				i;
	BIter				i1;
	BIter				i2;

	printf("Deleting Test MetaData\n");
	
	sel.startTime.setFirst();
	sel.endTime.setLast();
	sel.channels.append(SelectionChannel("TT", "TS.*"));

	// Delete Responses
	if(err = bds.responseGetList(sel, responses))
		return err;

	for(responses.start(i); !responses.isEnd(i); responses.next(i)){
		if(err = bds.responseDelete(responses[i].id))
			return err;
	}

	// Delete calibrations
	if(err = bds.calibrationGetList(sel, calibrations))
		return err;

	for(calibrations.start(i); !calibrations.isEnd(i); calibrations.next(i)){
		if(err = bds.calibrationDelete(calibrations[i].id))
			return err;
	}

	// Delete Channels
	if(err = bds.channelGetList(sel, channels))
		return err;

	for(channels.start(i); !channels.isEnd(i); channels.next(i)){
//		printf("Channels: %s\n", channels[i].channel.retStr());

		// Delete instruments
		sel.channelId = channels[i].id;
		if(err = bds.channelInstrumentGetList(sel, channelInstruments))
			return err;
		for(channelInstruments.start(i1); !channelInstruments.isEnd(i1); channelInstruments.next(i1)){
			if(err = bds.sensorDelete(channelInstruments[i1].sensorId))
				return err;
			if(err = bds.digitiserDelete(channelInstruments[i1].digitiserId))
				return err;
			if(err = bds.channelInstrumentDelete(channelInstruments[i1].id))
				return err;
		}
		
		if(err = bds.channelDelete(channels[i].id))
			return err;
	}		

	// Delete Locations
	if(err = bds.locationGetList(sel, locations))
		return err;

	for(locations.start(i); !locations.isEnd(i); locations.next(i)){
//		printf("Location: %s\n", locations[i].station.retStr());
		if(err = bds.locationDelete(locations[i].id))
			return err;
	}		

	// Delete Stations
	if(err = bds.stationGetList(sel, stations))
		return err;

	for(stations.start(i); !stations.isEnd(i); stations.next(i)){
//		printf("Stations: %s\n", stations[i].name.retStr());
		if(err = bds.stationDelete(stations[i].id))
			return err;
	}		

	// Delete Networks
	if(err = bds.networkGetList(networks))
		return err;

	for(networks.start(i); !networks.isEnd(i); networks.next(i)){
//		printf("Network: %s\n", networks[i].network.retStr());
		if(networks[i].network == "TT"){
			if(err = bds.networkDelete(networks[i].id))
				return err;
		}
	}		

	return err;
}

BError testAddMetaData(AdminAccess& bds, Config& config){
	BError			err;
	Network			network;
	Station			array;
	Station			station;
	Location		location;
	Channel			channel;
	Sensor			sensor;
	Digitiser		digitiser;
	ChannelInstrument	channelInstrument;
	Calibration		calibration;
	Response		response;
	BUInt			i;
	BUInt			w;
	BUInt32			id;
	
	printf("Creating Test MetaData\n");

	// Add Network
	network.network = "TT";
	network.description = "Test Network for BDS system tests";
	if(err = bds.networkUpdate(1, network, id))
		return err;
	
	for(w = 0; w < 4; w++){
		// Add Stations
		for(i = 0; i < 10; i++){
			switch(w){
			case 0:	station.name.printf("TSB%02.2d", i + 1);	break;
			case 1:	station.name.printf("TSR%02.2d", i + 1);	break;
			case 2:	station.name.printf("TSM%02.2d", i + 1);	break;
			case 3:	station.name.printf("TSN%02.2d", i + 1);	break;
			}
			station.alias = "";
			station.type = "station";
			station.description.printf("Test station: B%d", i + 1);;

			if(err = bds.stationUpdate(1, station, id))
				return err;
			
			location.startTime.setFirst();
			location.endTime.setString("2008-02-01T00:01:00");
			location.network = network.network;
			location.station = station.name;
			location.datum = "WGS84";
			if(w & 1){
				location.longitude = -4.5 + i;
				location.latitude =  0.0;
				location.arrayOffsetEast = -4.5 + i;
				location.arrayOffsetNorth = 0.0;
			}
			else {
				location.longitude = 0.0;
				location.latitude =  -4.5 + i;
				location.arrayOffsetEast = 0.0;
				location.arrayOffsetNorth = -4.5 + i;
			}
			location.elevation = 300.0;

			if(err = bds.locationUpdate(1, location, id))
				return err;

			location.startTime.setString("2008-02-01T00:01:00");
			location.endTime.setLast();
			location.network = network.network;
			location.station = station.name;
			location.datum = "WGS84";
			if(w & 1){
				location.longitude = -4.5 + i;
				location.latitude =  1.0;
				location.arrayOffsetEast = -4.5 + i;
				location.arrayOffsetNorth = 0.0;
			}
			else {
				location.longitude = 1.0;
				location.latitude =  -4.5 + i;
				location.arrayOffsetEast = 0.0;
				location.arrayOffsetNorth = -4.5 + i;
			}
			location.elevation = 300.0;

			if(err = bds.locationUpdate(1, location, id))
				return err;



			channel.startTime.setFirst();
			channel.endTime.setLast();
			channel.network = network.network;
			channel.station = station.name;
			channel.channel = "BHZ_01";
			channel.channelType = "BHZ";
			channel.channelAux = "01";
			channel.dataType = "seismic";
			channel.description = "Test Channel";
			if(err = bds.channelUpdate(1, channel, id))
				return err;

			channelInstrument.channelId = id;

			array.channels.append(ArrayChannel(station.name, channel.channel));
			

			
			// Add Calibration
			calibration.startTime.setFirst();
			calibration.endTime.setString("2008-02-01T00:02:00");
			calibration.network = network.network;
			calibration.station = station.name;
			calibration.channel = channel.channel;
			calibration.source = "Main";
			calibration.name = "Main";
			if(w > 1)
				calibration.samplingFrequency = 2.0;
			else
				calibration.samplingFrequency = 20.0;
			calibration.calibrationFrequency = 1.0;
			calibration.calibrationFactor = 1e-9;
			calibration.calibrationUnits = "m";
			calibration.depth = 10.1;
			calibration.horizontalAngle = 0.1;
			calibration.verticalAngle = 0.2;
			

			if(err = bds.calibrationUpdate(1, calibration, id))
				return err;

			// Add Calibration
			calibration.startTime.setString("2008-02-01T00:02:00");
			calibration.endTime.setString("2008-06-01T00:15:00");
			calibration.network = network.network;
			calibration.station = station.name;
			calibration.channel = channel.channel;
			calibration.source = "Main";
			if(w > 1)
				calibration.samplingFrequency = 2.0;
			else
				calibration.samplingFrequency = 20.0;
			calibration.calibrationFrequency = 1.0;
			calibration.calibrationFactor = 2e-9;
			calibration.calibrationUnits = "m";
			calibration.depth = 10.1;
			calibration.horizontalAngle = 0.0;
			calibration.verticalAngle = 90.0;

			if(err = bds.calibrationUpdate(1, calibration, id))
				return err;

			// Add Calibration
			calibration.startTime.setString("2008-06-01T00:15:00");
			calibration.endTime.setLast();
			calibration.network = network.network;
			calibration.station = station.name;
			calibration.channel = channel.channel;
			calibration.source = "Main";
			if(w > 1)
				calibration.samplingFrequency = 2.0;
			else
				calibration.samplingFrequency = 20.0;
			calibration.calibrationFrequency = 1.0;
			calibration.calibrationFactor = 3e-9;
			calibration.calibrationUnits = "m";
			calibration.depth = 10.1;
			calibration.horizontalAngle = 0.0;
			calibration.verticalAngle = 90.0;

			if(err = bds.calibrationUpdate(1, calibration, id))
				return err;

			// Add Response
			response.startTime.setFirst();
			response.endTime.setString("2008-02-01T00:03:00");
			response.network = network.network;
			response.station = station.name;;
			response.channel = channel.channel;
			response.source = "Main";
			response.name = "Sensor";
			response.type = "PoleZero";
			response.poleZeros.zeros.clear();
			response.poleZeros.poles.clear();
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(-3030.0, 0.0));
			response.poleZeros.poles.append(BComplex(-0.3950, 0.0000));
			response.poleZeros.poles.append(BComplex(-3.8330, 4.9790));
			response.poleZeros.poles.append(BComplex(-3.8330, -4.9790));
			response.poleZeros.poles.append(BComplex(-42.6800, 0.0000));

			if(err = bds.responseUpdate(1, response, id))
				return err;

			// Add Response
			response.startTime.setString("2008-02-01T00:03:00");
			response.endTime.setLast();
			response.network = network.network;
			response.station = station.name;;
			response.channel = channel.channel;
			response.source = "Main";
			response.name = "Sensor";
			response.type = "PoleZero";
			response.poleZeros.zeros.clear();
			response.poleZeros.poles.clear();
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(0.0, 0.0));
			response.poleZeros.zeros.append(BComplex(-3031.0, 0.0));
			response.poleZeros.poles.append(BComplex(-0.3950, 0.0000));
			response.poleZeros.poles.append(BComplex(-3.8330, 4.9790));
			response.poleZeros.poles.append(BComplex(-3.8330, -4.9790));
			response.poleZeros.poles.append(BComplex(-42.6800, 0.0000));

			if(err = bds.responseUpdate(1, response, id))
				return err;

			// Add Digitiser
			digitiser.startTime.setFirst();
			digitiser.endTime.setLast();
			digitiser.name = "TEST DIGITISER1";
			digitiser.type = "TEST TYPE";
			digitiser.serialNumber = BString::convert(w) + "-" + BString::convert(i);
			digitiser.numberChannels = 1;

			if(err = bds.digitiserUpdate(1, digitiser, channelInstrument.digitiserId))
				return err;	

			// Add Sensor
			sensor.startTime.setFirst();
			sensor.endTime.setLast();
			sensor.name = "TEST SENSOR1";
			sensor.type = "TEST TYPE";
			sensor.serialNumber = BString::convert(w) + "-" + BString::convert(i);
			sensor.numberChannels = 1;

			if(err = bds.sensorUpdate(1, sensor, channelInstrument.sensorId))
				return err;
				
			channelInstrument.startTime.setFirst();
			channelInstrument.endTime.setString("2008-02-01T00:04:00");
			channelInstrument.source = "Main";
			if(err = bds.channelInstrumentUpdate(1, channelInstrument, id))
				return err;
				

			// Add Digitiser
			digitiser.startTime.setFirst();
			digitiser.endTime.setLast();
			digitiser.name = "TEST DIGITISER2";
			digitiser.type = "TEST TYPE";
			digitiser.serialNumber = BString::convert(w) + "-" + BString::convert(i);
			digitiser.numberChannels = 1;

			if(err = bds.digitiserUpdate(1, digitiser, channelInstrument.digitiserId))
				return err;	

			// Add Sensor
			sensor.startTime.setFirst();
			sensor.endTime.setLast();
			sensor.name = "TEST SENSOR2";
			sensor.type = "TEST TYPE";
			sensor.serialNumber = BString::convert(w) + "-" + BString::convert(i);
			sensor.numberChannels = 1;

			if(err = bds.sensorUpdate(1, sensor, channelInstrument.sensorId))
				return err;
				
			channelInstrument.startTime.setString("2008-02-01T00:04:00");
			channelInstrument.endTime.setLast();
			channelInstrument.source = "Main";
			if(err = bds.channelInstrumentUpdate(1, channelInstrument, id))
				return err;

		}
	}
	
	// Add the Array
	array.name = "TSA";
	array.type = "array";
	array.description = "Test Array";
	if(err = bds.stationUpdate(1, array, id))
		return err;


	return err;
}



void usage(void) {
	cerr << "Usage: bdsTestCreateData [options]\n";
	cerr << " -help              - Help on command line parameters\n";
	cerr << " -host <hostname>   - BDS Server host name\n";
	cerr << " -user <user:password> - user id and password\n";
	cerr << " -verbose           - verbose\n";
	cerr << " -metadata          - fetch metadata rather than data\n";
}

static struct option options[] = {
		{ "?",			0, NULL, 0 },
		{ "h",			0, NULL, 0 },
		{ "help",		0, NULL, 0 },
		{ "host",		1, NULL, 0 },
		{ "user",		1, NULL, 0 },
		{ "verbose",		0, NULL, 0 },
		{ 0,0,0,0 }
};

int main(int argc, char** argv){
	BError			err;
	int			optIndex = 0;
	int			c;
	BString			s;
	AdminAccess		bds;
	BString			hostName;
	BString			user;
	BString			password;
	Config			config;
	
	// Set stdout to line buffering
	setlinebuf(stdout);

	hostName = getenv("BDS_HOST");
	if(hostName == "")
		hostName = "localhost";

	while((c = getopt_long_only(argc, argv, "", options, &optIndex)) == 0){
		s = options[optIndex].name;
		if(s == "help" || s == "h" || s == "?"){
			usage();
			return 1;
		}
		else if(s == "host"){
			hostName = optarg;
		}
		else if(s == "user"){
			s = optarg;
			user = s.pullToken(":");
			password = s;
		}
		else if(s == "verbose"){
			config.verbose = 1;
		}
		else {
			usage();
			return 1;
		}
	}
	
	if((argc - optind) != 0){
		usage();
		return 1;
	}

	// Open BDS access
	err = bds.connectService(BString("//") + hostName + "/bdsAdminAccess");

	// Connect to service
	if(!err)
		err = bds.connect(user, password);

	if(!err){
		err = testDeleteMetaData(bds, config);
	}
	
	if(!err){
		err = testAddMetaData(bds, config);
	}
	
	if(err)
		cerr << "Error: " << err.getString() << "\n";

	return err.getErrorNo();
}
